@@ -239,6 +239,14 @@ def get_keltner_lower_bands(EMA, ATR, n):
 		list.append(EMA - (i + 1) * 0.025 * ATR)
 	return list
 
+def get_keltner_max_upper_bands(EMA, ATR, n):
+	# Keltner Channel Upper Band = EMA + n*0.025*ATR, n from 1 to 100
+	return EMA + n * 0.025 * ATR
+
+def get_keltner_min_lower_bands(EMA, ATR, n):
+	# Keltner Channel Upper Band = EMA + n*0.025*ATR, n from 1 to 100
+	return EMA - n * 0.025 * ATR
+
 
 def get_process(currency_pairs, db_path, total_run_time, aggregate_time):
 	# The api key given by the professor
@@ -408,12 +416,6 @@ def get_process_agg(currency, db_path, nullPeriod, total_run_time, aggregate_tim
 		# This gets the Last Trade object defined in the API Resource
 		last_trade = resp['last']
 
-		# Format the timestamp from the result
-		dt = ts_to_datetime(last_trade["timestamp"])
-
-		# Get the current time and format it
-		insert_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
-
 		# Calculate the price by taking the average of the bid and ask prices
 		avg_price = (last_trade['bid'] + last_trade['ask']) / 2
 
@@ -425,47 +427,63 @@ def get_process_agg(currency, db_path, nullPeriod, total_run_time, aggregate_tim
 		curVOL = curMax - curMin
 		print("curMin: \n", curMin)
 		print("curMax: \n", curMax)
-		print("curTotal: \n", curTotal)
-		print("curMean: \n", curMean)
+		# print("curTotal: \n", curTotal)
+		# print("curMean: \n", curMean)
 		print("curVOL: \n", curVOL)
 
-		# CAUTION: the curMin, curMax, curMean, curVOL will be updated from 0 to nullPeriod,
-		# but it doesn't aggect db since only insert in db after nullPeriod
 
 		# if can calculate keltner bands, calculate fd for each 6-minute bucket
+		# after inserting, means the new start of 6 minutes
 		if count > nullPeriod:
 			# each interval between bands are 0.025*ATR
 			# to know how many bands it crossed is to count the difference between curPrice and lastPrce is composed of how many intervals
-			interval = 0.025*curVOL
-			difference = max(avg_price - lastPrice, lastPrice - avg_price)  # need to be positive
-			fd += (difference / interval)
+			interval = 0.025 * ATR
+			if interval != 0:
+				larger = max(avg_price, lastPrice)
+				smaller = min(avg_price, lastPrice)
+				maxLine = min(get_keltner_max_upper_bands(EMA, ATR, 100), larger)
+				minLine = max(get_keltner_min_lower_bands(EMA, ATR, 100), smaller)
+				print("maxLine: ", maxLine)
+				print("minLine: ", minLine)
+				print("interval: \n", interval)
+				if int((maxLine - minLine) / interval) < 0:
+					print("smallerrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr")
+				fd += int((maxLine - minLine) / interval)
 
 		lastPrice = avg_price
 
-		# Make a check to see if 6 minutes has been reached or not
-		# if agg_count == 360:   # 20 for test
-		if count > nullPeriod and agg_count == aggregate_time:
-			# Aggregate the data and clear the raw data tables
-			lastMin = curMin
-			lastMax = curMax
-			EMA = curMean  # lastMean
-			ATR = curVOL
-			keltner_upper_bands = get_keltner_upper_bands(EMA, ATR, 100)
-			keltne_lower_bands = get_keltner_lower_bands(EMA, ATR, 100)
-			lastfd = fd
-			# Get the current time and format it
-			insert_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
+		if agg_count == aggregate_time:
+			# when it reaches the end of the bucket, update and reinitialize information for each bucket
+			if count < nullPeriod:
+				agg_count = 0
+			else:
+				# Aggregate the data and clear the raw data tables
+				lastMin = curMin
+				lastMax = curMax
+				EMA = curMean  # lastMean
+				ATR = curVOL
+				print("ATR: ", ATR)
+				keltner_upper_bands = get_keltner_upper_bands(EMA, ATR, 100)
+				keltne_lower_bands = get_keltner_lower_bands(EMA, ATR, 100)
+				# when count == nullPeriod, it means the end of the nullPeriod,
+				# only calculate EMA, ATR and bands for calculating fd for the next period
 
-			# add column (max, min, mean, vol, fd, inserttime) for each aggregate_time minutes
-			with engine.begin() as conn:
-				try:
-					conn.execute(text(
-						"INSERT INTO " + from_ + to + "_aggfive(max, min, mean, vol, fd, inserttime) VALUES (:max, :min, :mean, :vol, :fd, :inserttime)"),
-						[{"max": lastMax, "min": lastMin, "mean": EMA, "vol": ATR, "fd": lastfd,
-						  "inserttime": insert_time}])
-				#                     print("inserting")
-				except Exception as e:
-					print(e)
+				if count > nullPeriod:
+					# update and insert
+					lastfd = fd
+					# Get the current time and format it
+					insert_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
+					# add column (max, min, mean, vol, fd, inserttime) for each aggregate_time minutes
+					with engine.begin() as conn:
+						try:
+							print("inserting: max {0}, min {1}, mean {2}, vol {3}, fd {4}, inserttime {5}\n".format(lastMax, lastMin, EMA, ATR, lastfd, insert_time))
+							conn.execute(text(
+								"INSERT INTO " + from_ + to + "_aggfive(max, min, mean, vol, fd, inserttime) VALUES (:max, :min, :mean, :vol, :fd, :inserttime)"),
+								[{"max": lastMax, "min": lastMin, "mean": EMA, "vol": ATR, "fd": lastfd,
+								  "inserttime": insert_time}])
+						#                     print("inserting")
+						except Exception as e:
+							print(e)
 
 			# since we calculate for each bucket, we need to reinitialize
 			curMin = sys.float_info.max
